<?php
	$lang = array(
		"warning"            => "The firewall on this server is blocking your connection.",
		"contact"            => "You need to contact the server owner or hosting provider for further information.",
		"blocked ip"         => "Your blocked IP address is:",
		"hostname"           => "The hostname of this server is:",
		"recaptcha title"    => "You can try to unblock yourself using ReCAPTCHA:",
		"recaptcha note"     => "Please note: Not all unblock requests will be successful as it is dependent on how your IP address is being blocked. If the unblock fails you will need to contact the server owner or hosting provider for further information.",
		"recaptcha success"  => "Passed human test. Please wait a few seconds and your IP address should be unblocked and you can return to the website:",
		"recaptcha failure"  => "Failed to pass ReCAPTCHA test. Please try again.",
		"recaptcha hostfail" => "There has been a problem verifying the hostname:",
		"recaptcha error"    => "There has been a problem. Please click on the reCAPTCHA box.",
		"unblock submit"     => "Unblock",
	);
?>
